@@ -45,6 +45,7 @@
 #include <KSharedConfig>
 #include <KStatusNotifierItem>
 #include <KUiServerV2JobTracker>
+#include <KUriFilter>
 #include <kcoreaddons_version.h>
 // #include <KSwitchLanguageDialog>
 
@@ -68,6 +69,7 @@
 #include "PowerManagementInterface.h"
 #include "RefreshNotifier.h"
 #include "discoversettings.h"
+#include <resources/ResourcesUpdatesModel.h>
 
 using namespace Qt::StringLiterals;
 
@@ -89,14 +91,14 @@ static void scheduleShutdownWithErrorCode()
 DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     : QObject()
     , m_engine(new QQmlApplicationEngine)
-    , m_networkAccessManagerFactory(new CachedNetworkAccessManagerFactory)
+    , m_networkAccessManagerFactory(std::make_unique<CachedNetworkAccessManagerFactory>())
 {
     setObjectName(QStringLiteral("DiscoverMain"));
-    m_engine->rootContext()->setContextObject(new KLocalizedQmlContext(m_engine));
+    m_engine->rootContext()->setContextObject(new KLocalizedQmlContext(m_engine.get()));
     auto factory = m_engine->networkAccessManagerFactory();
     m_engine->setNetworkAccessManagerFactory(nullptr);
     delete factory;
-    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
+    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.get());
 
     new RefreshNotifier(this);
 
@@ -131,7 +133,7 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     auto uri = "org.kde.discover";
     DiscoverDeclarativePlugin *plugin = new DiscoverDeclarativePlugin;
     plugin->setParent(this);
-    plugin->initializeEngine(m_engine, uri);
+    plugin->initializeEngine(m_engine.get(), uri);
     plugin->registerTypes(uri);
 
     m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
@@ -146,7 +148,7 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     }
 
     {
-        QQmlComponent component(m_engine, discoverQmlUri, u"DiscoverWindow"_s);
+        QQmlComponent component(m_engine.get(), discoverQmlUri, u"DiscoverWindow"_s);
         auto object = component.beginCreate(m_engine->rootContext());
         auto window = qobject_cast<QQuickWindow *>(object);
         if (!object || !window) {
@@ -216,7 +218,7 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
         this);
 
     if (ResourcesModel::global()->backends().isEmpty()) {
-        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+        connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, action, &OneTimeAction::trigger);
     } else {
         action->trigger();
     }
@@ -224,6 +226,7 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
 
 DiscoverObject::~DiscoverObject()
 {
+    m_isDeleting = true;
     m_engine->deleteLater();
 }
 
@@ -285,7 +288,7 @@ void DiscoverObject::openCategory(const QString &category)
     showLoadingPage();
     auto action = new OneTimeAction(
         [this, category]() {
-            Category *cat = CategoryModel::global()->findCategoryByName(category);
+            std::shared_ptr<Category> cat = CategoryModel::global()->findCategoryByName(category);
             if (cat) {
                 Q_EMIT listCategoryInternal(cat);
             } else {
@@ -302,6 +305,13 @@ void DiscoverObject::openCategory(const QString &category)
     }
 }
 
+void DiscoverObject::startHeadlessUpdate()
+{
+    openMode(u"update"_s);
+    QQuickItem *updatesPage = m_mainWindow->property("leftPage").value<QQuickItem *>();
+    updatesPage->setProperty("startHeadlessUpdate", true);
+}
+
 void DiscoverObject::openLocalPackage(const QUrl &localfile)
 {
     if (!QFile::exists(localfile.toLocalFile())) {
@@ -362,6 +372,9 @@ void DiscoverObject::openApplication(const QUrl &url)
                     connect(timeout, &QTimer::timeout, timeout, &QTimer::deleteLater);
 
                     auto openResourceOrWait = [this, res, timeout] {
+                        if (m_isDeleting) {
+                            return false;
+                        }
                         auto idx = kIndexOf(res, [](auto res) {
                             return res.resource->isInstalled();
                         });
@@ -460,9 +473,7 @@ bool DiscoverObject::quitWhenIdle()
     }
 
     if (!m_sni) {
-        auto tracker = new KUiServerV2JobTracker(m_sni);
-
-        m_sni = new KStatusNotifierItem(this);
+        m_sni.reset(new KStatusNotifierItem);
         m_sni->setStatus(KStatusNotifierItem::Active);
         m_sni->setIconByName(QStringLiteral("plasmadiscover"));
         m_sni->setTitle(i18n("Discover"));
@@ -472,13 +483,14 @@ bool DiscoverObject::quitWhenIdle()
         m_sni->setStandardActionsEnabled(false);
 
         connect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
-        connect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
+        connect(m_sni.get(), &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
 
         auto job = new TransactionsJob;
-        job->setParent(m_sni);
+        job->setParent(m_sni.get());
+        auto tracker = new KUiServerV2JobTracker(m_sni.get());
         tracker->registerJob(job);
         job->start();
-        connect(m_sni, &KStatusNotifierItem::activateRequested, job, &TransactionsJob::cancel);
+        connect(m_sni.get(), &KStatusNotifierItem::activateRequested, job, &TransactionsJob::cancel);
 
         m_mainWindow->hide();
     }
@@ -487,16 +499,14 @@ bool DiscoverObject::quitWhenIdle()
 
 void DiscoverObject::restore()
 {
-    if (!m_sni || !m_mainWindow) {
-        return;
+    if (m_sni) {
+        disconnect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
+        disconnect(m_sni.get(), &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
+        m_sni.reset();
+    }
+    if (m_mainWindow) {
+        m_mainWindow->show();
     }
-
-    disconnect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
-    disconnect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
-
-    m_mainWindow->show();
-    m_sni->deleteLater();
-    m_sni = nullptr;
 }
 
 void DiscoverObject::reconsiderQuit()
@@ -505,7 +515,7 @@ void DiscoverObject::reconsiderQuit()
         return;
     }
 
-    m_sni->deleteLater();
+    m_sni.reset();
     // Let the job UI to finalise properly
     QTimer::singleShot(20, qGuiApp, &QCoreApplication::quit);
 }
@@ -631,6 +641,15 @@ void DiscoverObject::copyTextToClipboard(const QString text)
     clipboard->setText(text);
 }
 
+QUrl DiscoverObject::searchUrl(const QString searchText)
+{
+    KUriFilterData filterData(searchText);
+    if (KUriFilter::self()->filterSearchUri(filterData, KUriFilter::NormalTextFilter)) {
+        return filterData.uri();
+    }
+    return {};
+}
+
 void DiscoverObject::setAboutToPowerOff()
 {
     for (auto backend : ResourcesModel::global()->backends()) {
