@@ -14,6 +14,7 @@
 
 #include <ReviewsBackend/Rating.h>
 #include <Transaction/Transaction.h>
+#include <appstream/AppStreamConcurrentPool.h>
 #include <appstream/AppStreamUtils.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <resources/SourcesModel.h>
@@ -156,7 +157,7 @@ public:
         m_resources.insert(resource->uniqueId(), resource);
 
         QObject::connect(resource, &FlatpakResource::sizeChanged, m_backend, [this, resource] {
-            if (!m_backend->isFetching()) {
+            if (m_backend->m_isFetching == 0) {
                 Q_EMIT m_backend->resourcesChanged(resource, {"size", "sizeDescription"});
             }
         });
@@ -170,25 +171,25 @@ public:
     AppStream::ComponentBox componentsByName(const QString &name)
     {
         auto components = m_pool->componentsById(name);
-        if (!components.isEmpty()) {
-            return components;
+        if (!components.result().isEmpty()) {
+            return components.result();
         }
 
         components = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
-        return components;
+        return components.result();
     }
 
     AppStream::ComponentBox componentsByFlatpakId(const QString &ref)
     {
-        AppStream::ComponentBox components = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
-        if (!components.isEmpty())
-            return components;
+        auto components = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
+        if (!components.result().isEmpty())
+            return components.result();
 
         components = m_pool->componentsByProvided(AppStream::Provided::KindId, ref.section('/'_L1, 1, 1));
-        return components;
+        return components.result();
     }
 
-    AppStream::Pool *m_pool = nullptr;
+    std::unique_ptr<AppStream::ConcurrentPool> m_pool;
     QHash<FlatpakResource::Id, FlatpakResource *> m_resources;
 
 private:
@@ -417,24 +418,6 @@ private:
     const QUrl m_url;
 };
 
-FlatpakRemote *FlatpakBackend::getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const
-{
-    auto remotes = flatpak_installation_list_remotes(installation, m_cancellable, nullptr);
-    if (!remotes) {
-        return nullptr;
-    }
-
-    const auto comparableUrl = url.toUtf8();
-    for (uint i = 0; i < remotes->len; i++) {
-        auto remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
-
-        if (comparableUrl == flatpak_remote_get_url(remote)) {
-            return remote;
-        }
-    }
-    return nullptr;
-}
-
 FlatpakInstalledRef *FlatpakBackend::getInstalledRefForApp(const FlatpakResource *resource) const
 {
     Q_ASSERT(resource->resourceType() != FlatpakResource::Source);
@@ -578,6 +561,30 @@ QSharedPointer<FlatpakSource> FlatpakBackend::findSource(FlatpakInstallation *in
     return {};
 }
 
+QSharedPointer<FlatpakSource> FlatpakBackend::findSourceUrl(FlatpakInstallation *installation, const QString &url) const
+{
+    for (const auto &source : m_flatpakSources) {
+        if (!source->remote()) {
+            continue;
+        }
+        g_autofree char *remoteUrl = flatpak_remote_get_url(source->remote());
+        if (source->installation() == installation && QUtf8StringView(remoteUrl) == url) {
+            return source;
+        }
+    }
+    for (const auto &source : m_flatpakLoadingSources) {
+        if (!source->remote()) {
+            continue;
+        }
+        g_autofree char *remoteUrl = flatpak_remote_get_url(source->remote());
+        if (source->installation() == installation && QUtf8StringView(remoteUrl) == url) {
+            return source;
+        }
+    }
+    qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Could not find source by url:" << url;
+    return {};
+}
+
 FlatpakResource *FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) const
 {
     FlatpakResource *runtime = nullptr;
@@ -846,34 +853,22 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     g_autoptr(GError) error = nullptr;
 
     // If we already added the remote, just go with it
-    g_autoptr(FlatpakRemote) remote = flatpak_installation_get_remote_by_name(preferredInstallation(), remoteName.toUtf8().constData(), m_cancellable, &error);
-    if (remote) {
-        g_autofree char *remoteUrl = flatpak_remote_get_url(remote);
-        if (remote && QString::fromUtf8(remoteUrl) != refurl) {
-            remote = nullptr;
-        }
-    }
-    if (remote) {
-        Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(remote));
-        m_refreshAppstreamMetadataJobs.insert(remote);
-        if (auto source = integrateRemote(preferredInstallation(), remote)) {
-            const QString ref = composeRef(isRuntime, name, branch);
-            auto searchComponent = [this, stream, source, ref, remote] {
-                Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(remote));
-                auto components = source->componentsByFlatpakId(ref);
-                auto resources = kTransform<QVector<StreamResult>>(components, [this, source](const auto &component) {
-                    return resourceForComponent(component, source);
-                });
-                Q_EMIT stream->resourcesFound(resources);
-                stream->finish();
-            };
-            if (source->m_pool) {
-                QTimer::singleShot(0, this, searchComponent);
-            } else {
-                connect(this, &FlatpakBackend::initialized, stream, searchComponent);
-            }
-            return;
+    if (auto source = findSourceUrl(preferredInstallation(), refurl)) {
+        const QString ref = composeRef(isRuntime, name, branch);
+        auto searchComponent = [this, stream, source, ref] {
+            auto components = source->componentsByFlatpakId(ref);
+            auto resources = kTransform<QVector<StreamResult>>(components, [this, source](const auto &component) {
+                return resourceForComponent(component, source);
+            });
+            Q_EMIT stream->resourcesFound(resources);
+            stream->finish();
+        };
+        if (source->m_pool) {
+            QTimer::singleShot(0, this, searchComponent);
+        } else {
+            connect(this, &FlatpakBackend::initialized, stream, searchComponent);
         }
+        return;
     }
 
     AppStream::Component asComponent = fetchComponentFromRemote(settings, m_cancellable);
@@ -890,7 +885,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     resource->setResourceFile(url);
     resource->setResourceLocation(QUrl(refurl));
     resource->setOrigin(remoteName);
-    resource->setDisplayOrigin(remote ? copyAndFree(flatpak_remote_get_title(remote)) : QString());
+    resource->setDisplayOrigin(remoteName);
     resource->setFlatpakName(name);
     resource->setArch(QString::fromUtf8(flatpak_get_default_arch()));
     resource->setBranch(branch);
@@ -1099,7 +1094,8 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCatalog);
 
     const auto loadDone = [this, source, pool](bool result) {
-        source->m_pool = pool;
+        source->m_pool.reset(new AppStream::ConcurrentPool);
+        source->m_pool->reset(pool, &m_threadPool);
         m_flatpakLoadingSources.removeAll(source);
         if (result) {
             m_flatpakSources += source;
@@ -1411,11 +1407,8 @@ void FlatpakBackend::acquireFetching(bool f)
         m_isFetching--;
     }
 
-    if ((!f && m_isFetching == 0) || (f && m_isFetching == 1)) {
-        Q_EMIT fetchingChanged();
-    }
-
-    if (m_isFetching == 0) {
+    if (!f && m_isFetching == 0) {
+        Q_EMIT contentsChanged();
         Q_EMIT initialized();
     }
 }
@@ -1460,7 +1453,7 @@ ResultsStream *FlatpakBackend::deferredResultStream(const QString &streamName, s
     // See https://devblogs.microsoft.com/oldnewthing/20211103-00/?p=105870
     [](FlatpakBackend *self, ResultsStream *stream, std::function<QCoro::Task<>(ResultsStream *)> callback) -> QCoro::Task<> {
         QPointer<ResultsStream> guard = stream;
-        if (self->isFetching()) {
+        if (self->m_isFetching > 0) {
             co_await qCoro(self, &FlatpakBackend::initialized);
         } else {
             co_await QCoro::sleepFor(0ms);
@@ -1478,6 +1471,32 @@ ResultsStream *FlatpakBackend::deferredResultStream(const QString &streamName, s
     return stream;
 }
 
+ResultsStream *FlatpakBackend::deferredResultStreamNoFinish(const QString &streamName, std::function<QCoro::Task<>(ResultsStream *)> callback)
+{
+    ResultsStream *stream = new ResultsStream(streamName);
+    stream->setParent(this);
+
+    // Don't capture variables into a coroutine lambda, pass them in as arguments instead
+    // See https://devblogs.microsoft.com/oldnewthing/20211103-00/?p=105870
+    [](FlatpakBackend *self, ResultsStream *stream, std::function<QCoro::Task<>(ResultsStream *)> callback) -> QCoro::Task<> {
+        QPointer<ResultsStream> guard = stream;
+        if (self->m_isFetching > 0) {
+            co_await qCoro(self, &FlatpakBackend::initialized);
+        } else {
+            co_await QCoro::sleepFor(0ms);
+        }
+        if (guard.isNull()) {
+            co_return;
+        }
+        co_await callback(stream);
+        if (guard.isNull()) {
+            co_return;
+        }
+    }(this, stream, std::move(callback));
+
+    return stream;
+}
+
 #define FLATPAK_BACKEND_GUARD                                                                                                                                  \
     QPointer<ResultsStream> guardStream(stream);                                                                                                               \
     g_autoptr(GCancellable) cancellable = g_object_ref(self->m_cancellable);                                                                                   \
@@ -1497,6 +1516,39 @@ static bool isFlatpakSubRef(const QLatin1String &name)
     return name.endsWith(QLatin1String(".Debug")) || name.endsWith(QLatin1String(".Locale")) || name.endsWith(QLatin1String(".Docs"));
 }
 
+void triage(FlatpakResource *resource, QVector<StreamResult> &prioritary, QVector<StreamResult> &rest, const AbstractResourcesBackend::Filters &filter)
+{
+    const bool matchById = resource->appstreamId().compare(filter.search, Qt::CaseInsensitive) == 0;
+    // Note: FlatpakResource can not have type == System
+    if (resource->type() == AbstractResource::ApplicationSupport && filter.state != AbstractResource::Upgradeable && !matchById) {
+        return;
+    }
+
+    if (resource->state() < filter.state) {
+        return;
+    }
+
+    if (!filter.extends.isEmpty() && !resource->extends().contains(filter.extends)) {
+        return;
+    }
+
+    if (!filter.mimetype.isEmpty() && !resource->mimetypes().contains(filter.mimetype)) {
+        return;
+    }
+
+    if (filter.search.isEmpty() || matchById) {
+        rest += resource;
+    } else if (resource->name().contains(filter.search, Qt::CaseInsensitive)) {
+        prioritary += resource;
+    } else if (resource->comment().contains(filter.search, Qt::CaseInsensitive)) {
+        rest += resource;
+        // trust The search terms provided by appstream are relevant, this makes possible finding "gimp"
+        // since the name() is "GNU Image Manipulation Program"
+    } else if (resource->appstreamId().contains(filter.search, Qt::CaseInsensitive)) {
+        rest += resource;
+    }
+}
+
 ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
     const auto fileName = filter.resourceUrl.fileName();
@@ -1698,90 +1750,62 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
         // Multithreading is nearly impossible for this stream, since child
         // objects are created and interact with this backend object. So the
         // task is split into hunks, interleaved by zero timers.
-        return deferredResultStream(u"FlatpakStream"_s, [this, filter](ResultsStream *stream) -> QCoro::Task<> {
+        return deferredResultStreamNoFinish(u"FlatpakStream"_s, [this, filter](ResultsStream *stream) -> QCoro::Task<> {
             return [](FlatpakBackend *self, ResultsStream *stream, const AbstractResourcesBackend::Filters filter) -> QCoro::Task<> {
                 FLATPAK_BACKEND_GUARD
                 const auto flatpakSources = self->m_flatpakSources;
                 QVector<StreamResult> prioritary, rest;
+                QMap<QSharedPointer<FlatpakSource>, QFuture<AppStream::ComponentBox>> futures;
+                QList<FlatpakResource *> unpooled;
 
                 for (const auto &source : flatpakSources) {
-                    FLATPAK_BACKEND_YIELD;
-
-                    QList<FlatpakResource *> resources;
                     if (source->m_pool) {
-                        const auto components = co_await [](const auto &filter, const auto &source) -> QCoro::Task<AppStream::ComponentBox> {
-                            if (!filter.search.isEmpty()) {
-                                co_return source->m_pool->search(filter.search);
-                            }
-                            if (filter.category) {
-                                co_return co_await AppStreamUtils::componentsByCategoriesTask(source->m_pool, filter.category, AppStream::Bundle::KindFlatpak);
-                            }
-                            co_return source->m_pool->components();
-                        }(filter, source);
-
-                        resources.reserve(components.size());
-                        for (const auto &component : components) {
-                            FLATPAK_BACKEND_YIELD;
-
-                            resources += self->resourceForComponent(component, source);
+                        if (!filter.search.isEmpty()) {
+                            futures.insert(source, source->m_pool->search(filter.search));
+                        } else if (filter.category) {
+                            futures.insert(source,
+                                           AppStreamUtils::componentsByCategoriesTask(source->m_pool.get(), filter.category, AppStream::Bundle::KindFlatpak));
+                        } else {
+                            futures.insert(source, source->m_pool->components());
                         }
                     } else {
-                        resources = source->m_resources.values();
-                    }
-
-                    for (auto resource : std::as_const(resources)) {
-                        FLATPAK_BACKEND_YIELD;
-
-                        const bool matchById = resource->appstreamId().compare(filter.search, Qt::CaseInsensitive) == 0;
-                        // Note: FlatpakResource can not have type == System
-                        if (resource->type() == AbstractResource::ApplicationSupport && filter.state != AbstractResource::Upgradeable && !matchById) {
-                            continue;
-                        }
-
-                        if (resource->state() < filter.state) {
-                            continue;
-                        }
-
-                        if (!filter.extends.isEmpty() && !resource->extends().contains(filter.extends)) {
-                            continue;
-                        }
-
-                        if (!filter.mimetype.isEmpty() && !resource->mimetypes().contains(filter.mimetype)) {
-                            continue;
-                        }
-
-                        if (filter.search.isEmpty() || matchById) {
-                            rest += resource;
-                        } else if (resource->name().contains(filter.search, Qt::CaseInsensitive)) {
-                            prioritary += resource;
-                        } else if (resource->comment().contains(filter.search, Qt::CaseInsensitive)) {
-                            rest += resource;
-                            // trust The search terms provided by appstream are relevant, this makes possible finding "gimp"
-                            // since the name() is "GNU Image Manipulation Program"
-                        } else if (resource->appstreamId().contains(filter.search, Qt::CaseInsensitive)) {
-                            rest += resource;
-                        }
+                        unpooled += source->m_resources.values();
                     }
                 }
-                auto f = [self](auto left, auto right) {
-                    return self->flatpakResourceLessThan(left, right);
-                };
-
-                // Even sorting can not be performed in other thread, and it can take a while
-                std::sort(rest.begin(), rest.end(), f);
-
-                FLATPAK_BACKEND_YIELD;
-
-                std::sort(prioritary.begin(), prioritary.end(), f);
-
-                QList<StreamResult> resources;
-                resources.reserve(prioritary.size() + rest.size());
-                resources.append(std::move(prioritary));
-                resources.append(std::move(rest));
-
-                if (!resources.isEmpty()) {
-                    Q_EMIT stream->resourcesFound(resources);
+                using QFS = QFutureWatcher<QList<QFuture<AppStream::ComponentBox>>>;
+                auto *fw = new QFS(stream);
+                fw->setFuture(QtFuture::whenAll(futures.begin(), futures.end()));
+                FLATPAK_BACKEND_YIELD
+                for (auto r : unpooled) {
+                    triage(r, prioritary, rest, filter);
                 }
+                connect(fw, &QFS::finished, stream, [self, futures, stream, _rest = rest, _prioritary = prioritary, filter, fw]() {
+                    QVector<StreamResult> rest(_rest), prioritary(_prioritary);
+                    for (const auto [source, future] : futures.asKeyValueRange()) {
+                        for (const auto &component : future.result()) {
+                            triage(self->resourceForComponent(component, source), prioritary, rest, filter);
+                        }
+                    }
+
+                    auto f = [self](auto left, auto right) {
+                        return self->flatpakResourceLessThan(left, right);
+                    };
+
+                    // Even sorting can not be performed in other thread, and it can take a while
+                    std::sort(rest.begin(), rest.end(), f);
+                    std::sort(prioritary.begin(), prioritary.end(), f);
+
+                    QList<StreamResult> resources;
+                    resources.reserve(prioritary.size() + rest.size());
+                    resources.append(std::move(prioritary));
+                    resources.append(std::move(rest));
+
+                    if (!resources.isEmpty()) {
+                        Q_EMIT stream->resourcesFound(resources);
+                    }
+                    stream->finish();
+                    fw->deleteLater();
+                });
             }(this, stream, filter);
         });
     }
@@ -1890,7 +1914,7 @@ ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
                 stream->finish();
             };
 
-            if (isFetching()) {
+            if (m_isFetching > 0) {
                 connect(this, &FlatpakBackend::initialized, stream, f);
             } else {
                 QTimer::singleShot(0, this, f);
@@ -2009,16 +2033,17 @@ Transaction *FlatpakBackend::installApplication(AbstractResource *app, const Add
                 auto source = findSource(resource->installation(), resource->origin());
                 if (!source) {
                     // It could mean that it's still integrating after checkRepositories It should update itself
-                    return;
-                }
-                resource->setTemporarySource({});
-                const auto id = resource->uniqueId();
-                source->m_resources.insert(id, resource);
+                    loadAppsFromAppstreamData(resource->installation());
+                } else {
+                    resource->setTemporarySource({});
+                    const auto id = resource->uniqueId();
+                    source->m_resources.insert(id, resource);
 
-                tempSource->m_resources.remove(id);
-                if (tempSource->m_resources.isEmpty()) {
-                    const bool removed = m_flatpakSources.removeAll(tempSource) || m_flatpakLoadingSources.removeAll(tempSource);
-                    Q_ASSERT(removed);
+                    tempSource->m_resources.remove(id);
+                    if (tempSource->m_resources.isEmpty()) {
+                        const bool removed = m_flatpakSources.removeAll(tempSource) || m_flatpakLoadingSources.removeAll(tempSource);
+                        Q_ASSERT(removed);
+                    }
                 }
             }
             updateAppState(resource);
@@ -2111,7 +2136,7 @@ InlineMessage *FlatpakBackend::explainDysfunction() const
 bool FlatpakBackend::extends(const QString &extends) const
 {
     return std::any_of(m_flatpakSources.constBegin(), m_flatpakSources.constEnd(), [extends](const auto &source) {
-        return source->m_pool && source->m_pool->lastError().isEmpty() && !source->m_pool->componentsByExtends(extends).isEmpty();
+        return source->m_pool && source->m_pool->lastError().isEmpty() && !source->m_pool->componentsByExtends(extends).result().isEmpty();
     });
 }
 
