@@ -19,6 +19,11 @@ public:
     QString brief() const;
     QString description() const;
 
+    bool operator==(const FlatpakPermission &other) const
+    {
+        return m_icon == other.m_icon && m_description == other.m_description && m_brief == other.m_brief;
+    }
+
 private:
     QString m_brief;
     QString m_description;
@@ -39,7 +44,7 @@ public:
 
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     QVariant data(const QModelIndex &index, int role) const override;
-    virtual QHash<int, QByteArray> roleNames() const override;
+    QHash<int, QByteArray> roleNames() const override;
 
 private:
     QVector<FlatpakPermission> m_permissions;
