@@ -20,6 +20,7 @@
 #include <KNSCore/Question>
 #include <KNSCore/QuestionManager>
 #include <KNSCore/ResultsStream>
+#include <KWindowSystem>
 
 // DiscoverCommon includes
 #include "Category/Category.h"
@@ -41,7 +42,7 @@ static const int ENGINE_PAGE_SIZE = 100;
 class KNSBackendFactory : public AbstractResourcesBackendFactory
 {
     Q_OBJECT
-    Q_PLUGIN_METADATA(IID "org.kde.muon.AbstractResourcesBackendFactory")
+    Q_PLUGIN_METADATA(IID DISCOVER_PLUGIN_IID)
     Q_INTERFACES(AbstractResourcesBackendFactory)
 public:
     KNSBackendFactory()
@@ -154,6 +155,37 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     setName(fileName);
     setObjectName(knsrc);
 
+    static const QSet<QStringView> knsrcPlasma = []() {
+        QSet<QStringView> ret = {
+            u"aurorae.knsrc",       u"icons.knsrc",           u"kfontinst.knsrc", u"lookandfeel.knsrc",
+            u"plasma-themes.knsrc", u"plasmoids.knsrc",       u"wallpaper.knsrc", u"wallpaper-mobile.knsrc",
+            u"xcursor.knsrc",       u"cgcgtk3.knsrc",         u"cgcicon.knsrc",   u"cgctheme.knsrc",
+            u"comic.knsrc",         u"colorschemes.knsrc",    u"emoticons.knsrc", u"plymouth.knsrc",
+            u"sddmtheme.knsrc",     u"wallpaperplugin.knsrc", u"ksplash.knsrc",
+        };
+
+        if (KWindowSystem::isPlatformX11()) {
+            ret.insert(u"kwinswitcher-x11.knsrc");
+            ret.insert(u"kwineffect-x11.knsrc");
+            ret.insert(u"kwinscripts-x11.knsrc");
+            ret.insert(u"window-decorations-x11.knsrc");
+        } else {
+            ret.insert(u"kwinswitcher.knsrc");
+            ret.insert(u"kwineffect.knsrc");
+            ret.insert(u"kwinscripts.knsrc");
+            ret.insert(u"window-decorations.knsrc");
+        }
+
+        return ret;
+    }();
+
+    const bool isPlasmaCategory = knsrcPlasma.contains(fileName);
+    static const bool weAreOnPlasma = qgetenv("XDG_CURRENT_DESKTOP") == "KDE";
+    if (isPlasmaCategory && !weAreOnPlasma) {
+        markInvalid(QStringLiteral("Skipping Plasma categories outside of Plasma."));
+        return;
+    }
+
     const KConfig conf(m_name, KConfig::SimpleConfig);
     const bool hasVersionlessGrp = conf.hasGroup(u"KNewStuff"_s);
     if (!conf.hasGroup(u"KNewStuff3"_s) && !hasVersionlessGrp) {
@@ -189,18 +221,28 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     m_hasApplications = group.readEntry<bool>("X-Discover-HasApplications", false);
 
     const QStringList cats = group.readEntry<QStringList>("Categories", QStringList{});
-    QVector<Category *> categories;
+    QList<std::shared_ptr<Category>> categories;
     if (cats.count() > 1) {
         m_categories += cats;
         for (const auto &cat : cats) {
             if (m_hasApplications)
-                categories << new Category(cat, QStringLiteral("applications-other"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
+                categories << std::make_shared<Category>(cat,
+                                                         QStringLiteral("applications-other"),
+                                                         CategoryFilter{CategoryFilter::CategoryNameFilter, cat},
+                                                         backendName,
+                                                         QList<std::shared_ptr<Category>>{},
+                                                         true);
             else
-                categories << new Category(cat, QStringLiteral("plasma"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
+                categories << std::make_shared<Category>(cat,
+                                                         QStringLiteral("plasma"),
+                                                         CategoryFilter{CategoryFilter::CategoryNameFilter, cat},
+                                                         backendName,
+                                                         QList<std::shared_ptr<Category>>{},
+                                                         true);
         }
     }
 
-    QVector<Category *> topCategories{categories};
+    QList<std::shared_ptr<Category>> topCategories{categories};
     for (const auto &cat : std::as_const(categories)) {
         const QString catName = cat->name().append(QLatin1Char('/'));
         for (const auto &potentialSubCat : std::as_const(categories)) {
@@ -219,7 +261,7 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
 
     connect(m_engine, &KNSCore::EngineBase::signalCategoriesMetadataLoaded, this, [categories](const QList<KNSCore::CategoryMetadata> &categoryMetadatas) {
         for (const KNSCore::CategoryMetadata &category : categoryMetadatas) {
-            for (Category *cat : std::as_const(categories)) {
+            for (const std::shared_ptr<Category> &cat : std::as_const(categories)) {
                 if (cat->matchesCategoryName(category.name())) {
                     cat->setName(category.displayName());
                     break;
@@ -230,14 +272,14 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     m_engine->init(m_name);
 
     if (m_hasApplications) {
-        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filter, backendName, topCategories, false);
-        auto applicationCategory = new Category(i18n("Applications"), //
-                                                QStringLiteral("applications-internet"),
-                                                filter,
-                                                backendName,
-                                                {actualCategory},
-                                                false);
-        const QVector<CategoryFilter> filters = {{CategoryFilter::CategoryNameFilter, QLatin1String("Application")}, filter};
+        auto actualCategory = std::make_shared<Category>(m_displayName, QStringLiteral("applications-other"), filter, backendName, topCategories, false);
+        std::shared_ptr<Category> applicationCategory = std::make_shared<Category>(i18n("Applications"), //
+                                                                                   QStringLiteral("applications-internet"),
+                                                                                   filter,
+                                                                                   backendName,
+                                                                                   QList<std::shared_ptr<Category>>{actualCategory},
+                                                                                   false);
+        const QList<CategoryFilter> filters = {{CategoryFilter::CategoryNameFilter, QLatin1String("Application")}, filter};
         applicationCategory->setFilter({CategoryFilter::AndFilter, filters});
         m_categories.append(applicationCategory->name());
         m_rootCategories = {applicationCategory};
@@ -257,28 +299,11 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         }
         m_engine->setTagFilter(tagFilter);
     } else {
-        static const QSet<QString> knsrcPlasma = {
-            QStringLiteral("aurorae.knsrc"),       QStringLiteral("icons.knsrc"),
-            QStringLiteral("kfontinst.knsrc"),     QStringLiteral("lookandfeel.knsrc"),
-            QStringLiteral("plasma-themes.knsrc"), QStringLiteral("plasmoids.knsrc"),
-            QStringLiteral("wallpaper.knsrc"),     QStringLiteral("wallpaper-mobile.knsrc"),
-            QStringLiteral("xcursor.knsrc"),
+        const auto iconName = isPlasmaCategory ? QStringLiteral("plasma") : QStringLiteral("applications-other");
+        auto actualCategory = std::make_shared<Category>(m_displayName, iconName, filter, backendName, categories, true);
 
-            QStringLiteral("cgcgtk3.knsrc"),       QStringLiteral("cgcicon.knsrc"),
-            QStringLiteral("cgctheme.knsrc"), // GTK integration
-            QStringLiteral("kwinswitcher.knsrc"),  QStringLiteral("kwineffect.knsrc"),
-            QStringLiteral("kwinscripts.knsrc"), // KWin
-            QStringLiteral("comic.knsrc"),         QStringLiteral("colorschemes.knsrc"),
-            QStringLiteral("emoticons.knsrc"),     QStringLiteral("plymouth.knsrc"),
-            QStringLiteral("sddmtheme.knsrc"),     QStringLiteral("wallpaperplugin.knsrc"),
-            QStringLiteral("ksplash.knsrc"),       QStringLiteral("window-decorations.knsrc"),
-        };
-        const auto iconName = knsrcPlasma.contains(fileName) ? QStringLiteral("plasma") : QStringLiteral("applications-other");
-        auto actualCategory = new Category(m_displayName, iconName, filter, backendName, categories, true);
-        actualCategory->setParent(this);
-
-        const auto topLevelName = knsrcPlasma.contains(fileName) ? i18n("Plasma Addons") : i18n("Application Addons");
-        auto addonsCategory = new Category(topLevelName, iconName, filter, backendName, {actualCategory}, true);
+        const auto topLevelName = isPlasmaCategory ? i18n("Plasma Addons") : i18n("Application Addons");
+        auto addonsCategory = std::make_shared<Category>(topLevelName, iconName, filter, backendName, QList<std::shared_ptr<Category>>{actualCategory}, true);
         m_rootCategories = {addonsCategory};
     }
 
@@ -287,7 +312,6 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
 
 KNSBackend::~KNSBackend()
 {
-    qDeleteAll(m_rootCategories);
 }
 
 void KNSBackend::markInvalid(const QString &message)
@@ -311,10 +335,9 @@ void KNSBackend::setFetching(bool f)
 {
     if (m_fetching != f) {
         m_fetching = f;
-        Q_EMIT fetchingChanged();
-
         if (!m_fetching) {
             Q_EMIT initialized();
+            Q_EMIT contentsChanged();
         }
     }
 }
@@ -479,7 +502,7 @@ static ResultsStream *voidStream()
 template<typename T>
 void KNSBackend::deferredResultStream(KNSResultsStream *stream, T start)
 {
-    if (isFetching()) {
+    if (m_fetching) {
         auto startOnce = [stream, start] {
             if (stream->hasStarted()) {
                 return;
@@ -489,7 +512,6 @@ void KNSBackend::deferredResultStream(KNSResultsStream *stream, T start)
 
         // If it's not ready to take queries, wait a bit longer
         connect(this, &KNSBackend::initialized, stream, startOnce, Qt::QueuedConnection);
-        connect(this, &KNSBackend::fetchingChanged, stream, startOnce, Qt::QueuedConnection);
     } else {
         QTimer::singleShot(0, stream, start);
     }
@@ -512,11 +534,10 @@ ResultsStream *KNSBackend::search(const AbstractResourcesBackend::Filters &filte
         };
         deferredResultStream(stream, start);
         return stream;
-    } else if ((!filter.category && !filter.search.isEmpty()) // Accept global searches
-                                                              // If there /is/ a category, make sure we actually are one of those requested before searching
-               || (filter.category && kContains(m_categories, [&filter](const QString &cat) {
-                       return filter.category->matchesCategoryName(cat);
-                   }))) {
+        // Make sure we actually are one of the requested categories before searching
+    } else if ((filter.category && kContains(m_categories, [&filter](const QString &cat) {
+                    return filter.category->matchesCategoryName(cat);
+                }))) {
         return searchStream(filter.search);
     }
     return voidStream();
@@ -526,7 +547,7 @@ KNSResultsStream *KNSBackend::searchStream(const QString &searchText)
 {
     auto stream = new KNSResultsStream(this, QLatin1String("KNS-search-") + name());
     auto start = [this, stream, searchText]() {
-        Q_ASSERT(!isFetching());
+        Q_ASSERT(!m_fetching);
         if (!m_isValid) {
             qWarning() << "querying an invalid backend";
             stream->finish();
@@ -560,11 +581,6 @@ ResultsStream *KNSBackend::findResourceByPackageName(const QUrl &search)
     return stream;
 }
 
-bool KNSBackend::isFetching() const
-{
-    return m_fetching;
-}
-
 AbstractBackendUpdater *KNSBackend::backendUpdater() const
 {
     return m_updater;
