@@ -8,7 +8,7 @@
 #include "KNSBackend.h"
 #include "KNSResource.h"
 #include <KLocalizedString>
-#include <KNSCore/Engine>
+#include <KNSCore/EngineBase>
 #include <KPasswordDialog>
 #include <QDebug>
 #include <QDesktopServices>
@@ -18,56 +18,13 @@
 #include <attica/providermanager.h>
 #include <resources/AbstractResource.h>
 
-KNSReviews::KNSReviews(KNSBackend *backend)
-    : AbstractReviewsBackend(backend)
-    , m_backend(backend)
-{
-}
-
-Rating *KNSReviews::ratingForApplication(AbstractResource *app) const
-{
-    KNSResource *resource = qobject_cast<KNSResource *>(app);
-    if (!resource) {
-        qDebug() << app->packageName() << "<= couldn't find resource";
-        return nullptr;
-    }
-
-    return resource->ratingInstance();
-}
-
-void KNSReviews::fetchReviews(AbstractResource *app, int page)
-{
-    Attica::ListJob<Attica::Comment> *job = provider().requestComments(Attica::Comment::ContentComment, app->packageName(), QStringLiteral("0"), page - 1, 10);
-    if (!job) {
-        Q_EMIT reviewsReady(app, {}, false);
-        return;
-    }
-    job->setProperty("app", QVariant::fromValue<AbstractResource *>(app));
-    connect(job, &Attica::BaseJob::finished, this, &KNSReviews::commentsReceived);
-    job->start();
-    acquireFetching(true);
-}
-
-void KNSReviews::acquireFetching(bool f)
-{
-    if (f)
-        m_fetching++;
-    else
-        m_fetching--;
-
-    if ((!f && m_fetching == 0) || (f && m_fetching == 1)) {
-        Q_EMIT fetchingChanged(m_fetching != 0);
-    }
-    Q_ASSERT(m_fetching >= 0);
-}
-
-static QVector<ReviewPtr> createReviewList(AbstractResource *app, const Attica::Comment::List comments, int depth = 0)
+static QVector<ReviewPtr> createReviewList(AbstractResource *resource, const Attica::Comment::List comments, int depth = 0)
 {
     QVector<ReviewPtr> reviews;
     for (const Attica::Comment &comment : comments) {
         // TODO: language lookup?
-        ReviewPtr r(new Review(app->name(),
-                               app->packageName(),
+        ReviewPtr r(new Review(resource->name(),
+                               resource->packageName(),
                                QStringLiteral("en"),
                                comment.subject(),
                                comment.text(),
@@ -78,25 +35,62 @@ static QVector<ReviewPtr> createReviewList(AbstractResource *app, const Attica::
                                comment.score() / 10,
                                0,
                                0,
+                               0.0,
                                QString()));
         r->addMetadata(QStringLiteral("NumberOfParents"), depth);
         reviews += r;
         if (comment.childCount() > 0) {
-            reviews += createReviewList(app, comment.children(), depth + 1);
+            reviews += createReviewList(resource, comment.children(), depth + 1);
         }
     }
     return reviews;
 }
 
-void KNSReviews::commentsReceived(Attica::BaseJob *j)
+KNSReviews::KNSReviews(KNSBackend *backend)
+    : AbstractReviewsBackend(backend)
+    , m_backend(backend)
 {
-    acquireFetching(false);
-    Attica::ListJob<Attica::Comment> *job = static_cast<Attica::ListJob<Attica::Comment> *>(j);
+}
 
-    AbstractResource *app = job->property("app").value<AbstractResource *>();
-    QVector<ReviewPtr> reviews = createReviewList(app, job->itemList());
+Rating KNSReviews::ratingForApplication(AbstractResource *resource) const
+{
+    if (resource) {
+        return resource->rating();
+    }
+    return {};
+}
 
-    Q_EMIT reviewsReady(app, reviews, !reviews.isEmpty());
+ReviewsJob *KNSReviews::fetchReviews(AbstractResource *resource, int page)
+{
+    const auto job = provider().requestComments(Attica::Comment::ContentComment, resource->packageName(), QStringLiteral("0"), page - 1, 10);
+    if (!job) {
+        auto r = new ReviewsJob;
+        r->deleteLater();
+        return r;
+    }
+    auto reviewJob = new ReviewsJob;
+    connect(job, &Attica::BaseJob::finished, this, [job, resource, reviewJob] {
+        QVector<ReviewPtr> reviews = createReviewList(resource, job->itemList());
+
+        Q_EMIT reviewJob->reviewsReady(reviews, !reviews.isEmpty());
+        reviewJob->deleteLater();
+    });
+    job->start();
+    return reviewJob;
+}
+
+void KNSReviews::acquireFetching(bool fetching)
+{
+    if (fetching) {
+        m_fetching++;
+    } else {
+        m_fetching--;
+    }
+
+    if ((!fetching && m_fetching == 0) || (fetching && m_fetching == 1)) {
+        Q_EMIT fetchingChanged(m_fetching != 0);
+    }
+    Q_ASSERT(m_fetching >= 0);
 }
 
 bool KNSReviews::isFetching() const
@@ -114,24 +108,31 @@ void KNSReviews::deleteReview(Review * /*r*/)
     qWarning() << "cannot delete comments";
 }
 
-void KNSReviews::sendReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating, const QString &userName)
+ReviewsJob *
+KNSReviews::sendReview(AbstractResource *resource, const QString &summary, const QString &reviewText, const QString &rating, const QString &userName)
 {
     Q_UNUSED(userName);
-    provider().voteForContent(app->packageName(), rating.toUInt() * 20);
-    if (!summary.isEmpty())
-        provider().addNewComment(Attica::Comment::ContentComment, app->packageName(), QString(), QString(), summary, review_text);
+    provider().voteForContent(resource->packageName(), rating.toUInt() * 20);
+    if (!summary.isEmpty()) {
+        provider().addNewComment(Attica::Comment::ContentComment, resource->packageName(), QString(), QString(), summary, reviewText);
+    }
+
+    auto job = new ReviewsJob;
+    job->deleteLater();
+    return job;
 }
 
-void KNSReviews::submitUsefulness(Review *r, bool useful)
+void KNSReviews::submitUsefulness(Review *review, bool useful)
 {
-    provider().voteForComment(QString::number(r->id()), useful * 5);
+    provider().voteForComment(QString::number(review->id()), useful * 5);
 }
 
 void KNSReviews::logout()
 {
-    bool b = provider().saveCredentials(QString(), QString());
-    if (!b)
+    bool ok = provider().saveCredentials(QString(), QString());
+    if (!ok) {
         qWarning() << "couldn't log out";
+    }
 }
 
 void KNSReviews::registerAndLogin()
@@ -141,16 +142,17 @@ void KNSReviews::registerAndLogin()
 
 void KNSReviews::login()
 {
-    KPasswordDialog *dialog = new KPasswordDialog;
+    const auto dialog = new KPasswordDialog;
     dialog->setPrompt(i18n("Log in information for %1", provider().name()));
     connect(dialog, &KPasswordDialog::gotUsernameAndPassword, this, &KNSReviews::credentialsReceived);
 }
 
 void KNSReviews::credentialsReceived(const QString &user, const QString &password)
 {
-    bool b = provider().saveCredentials(user, password);
-    if (!b)
+    bool ok = provider().saveCredentials(user, password);
+    if (!ok) {
         qWarning() << "couldn't save" << user << "credentials for" << provider().name();
+    }
 }
 
 bool KNSReviews::hasCredentials() const
@@ -173,7 +175,9 @@ Attica::Provider KNSReviews::provider() const
     return *m_backend->engine()->atticaProviders().constFirst();
 }
 
-bool KNSReviews::isResourceSupported(AbstractResource *res) const
+bool KNSReviews::isResourceSupported(AbstractResource *resource) const
 {
-    return qobject_cast<KNSResource *>(res);
+    return qobject_cast<KNSResource *>(resource);
 }
+
+#include "moc_KNSReviews.cpp"
