@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef PKRESOLVETRANSACTION_H
-#define PKRESOLVETRANSACTION_H
+#pragma once
 
 #include <PackageKit/Transaction>
 #include <QObject>
@@ -35,5 +34,3 @@ private:
     QVector<PackageKit::Transaction *> m_transactions;
     PackageKitBackend *const m_backend;
 };
-
-#endif
