@@ -128,7 +128,7 @@ Delay::Delay()
 
 PackageKitBackend::PackageKitBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
-    , m_appdata(new AppStream::Pool)
+    , m_appdata(new AppStream::ConcurrentPool)
     , m_updater(new PackageKitUpdater(this))
     , m_refresher(nullptr)
     , m_isFetching(0)
@@ -291,11 +291,6 @@ void PackageKitBackend::updateProxy()
     }
 }
 
-bool PackageKitBackend::isFetching() const
-{
-    return m_isFetching;
-}
-
 void PackageKitBackend::acquireFetching(bool f)
 {
     if (f) {
@@ -304,11 +299,10 @@ void PackageKitBackend::acquireFetching(bool f)
         m_isFetching--;
     }
 
-    if ((!f && m_isFetching == 0) || (f && m_isFetching == 1)) {
-        Q_EMIT fetchingChanged();
-        if (m_isFetching == 0) {
-            Q_EMIT available();
-        }
+    if (!f && m_isFetching == 0) {
+        Q_EMIT contentsChanged();
+        Q_EMIT available();
+        Q_EMIT updatesCountChanged();
     }
     Q_ASSERT(m_isFetching >= 0);
 }
@@ -317,7 +311,7 @@ void PackageKitBackend::reloadPackageList()
 {
     acquireFetching(true);
 
-    m_appdata.reset(new AppStream::Pool);
+    m_appdata->reset(new AppStream::Pool, &m_threadPool);
 
     const auto loadDone = [this](bool correct) {
         if (!correct && m_packages.packages.isEmpty()) {
@@ -340,10 +334,10 @@ void PackageKitBackend::reloadPackageList()
         acquireFetching(false);
 
         const auto distroComponents = m_appdata->componentsById(AppStream::SystemInfo::currentDistroComponentId());
-        if (distroComponents.isEmpty()) {
+        if (distroComponents.result().isEmpty()) {
             qWarning() << "PackageKitBackend: No distro component found for" << AppStream::SystemInfo::currentDistroComponentId();
         }
-        for (const AppStream::Component &dc : distroComponents) {
+        for (const AppStream::Component &dc : distroComponents.result()) {
             const auto releases = dc.releasesPlain().entries();
             for (const auto &r : releases) {
                 int cmp = AppStream::Utils::vercmpSimple(r.version(), AppStreamIntegration::global()->osRelease()->versionId());
@@ -366,7 +360,7 @@ void PackageKitBackend::reloadPackageList()
         }
     };
 
-    connect(m_appdata.get(), &AppStream::Pool::loadFinished, this, [this, loadDone](bool success) {
+    connect(m_appdata.get(), &AppStream::ConcurrentPool::loadFinished, this, [this, loadDone](bool success) {
         m_appdataLoaded = true;
         if (!success) {
             qWarning() << "PackageKitBackend: Could not open the AppStream metadata pool" << m_appdata->lastError();
@@ -565,7 +559,7 @@ T PackageKitBackend::resourcesByPackageNames(const W &pkgnames) const
                 if (resource) {
                     ret += resource;
                 } else {
-                    ret += resourcesByComponents<T>(m_appdata->componentsByBundleId(AppStream::Bundle::KindPackage, pkg_name, false));
+                    ret += resourcesByComponents<T>(m_appdata->componentsByBundleId(AppStream::Bundle::KindPackage, pkg_name, false).result());
                 }
             }
         }
@@ -582,6 +576,7 @@ void PackageKitBackend::checkForUpdates()
 
     if (!m_refresher) {
         acquireFetching(true);
+        Q_EMIT m_updater->fetchingChanged();
         m_updater->clearDistroUpgrade();
         m_refresher = PackageKit::Daemon::refreshCache(false);
         // Limit the cache-age so that we actually download new caches if necessary
@@ -593,6 +588,7 @@ void PackageKitBackend::checkForUpdates()
             m_refresher = nullptr;
             fetchUpdates();
             acquireFetching(false);
+            Q_EMIT m_updater->fetchingChanged();
         });
     } else {
         qWarning() << "PackageKitBackend: Already resetting";
@@ -605,10 +601,10 @@ AppStream::ComponentBox PackageKitBackend::componentsById(const QString &id) con
 {
     Q_ASSERT(m_appstreamInitialized);
     auto comps = m_appdata->componentsById(id);
-    if (comps.isEmpty()) {
+    if (comps.result().isEmpty()) {
         comps = m_appdata->componentsByProvided(AppStream::Provided::KindId, id);
     }
-    return comps;
+    return comps.result();
 }
 
 static bool needsResolveFilter(const StreamResult &result)
@@ -623,6 +619,7 @@ private:
         : ResultsStream(name)
         , backend(backend)
     {
+        Q_ASSERT(QThread::currentThread() == backend->thread());
     }
 
     PKResultsStream(PackageKitBackend *backend, const QString &name, const QVector<StreamResult> &resources)
@@ -649,6 +646,7 @@ public:
         }
 
         Q_ASSERT(resources.size() == QSet(resources.constBegin(), resources.constEnd()).size());
+        Q_ASSERT(QThread::currentThread() == backend->thread());
         const auto toResolve = kFilter<QVector<StreamResult>>(resources, needsResolveFilter);
         if (!toResolve.isEmpty()) {
             auto transaction = backend->resolvePackages(kTransform<QStringList>(toResolve, [](const StreamResult &result) {
@@ -703,9 +701,10 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         return findResourceByPackageName(filter.resourceUrl);
     } else if (!filter.extends.isEmpty()) {
         return deferredResultStream(u"PackageKitStream-extends"_s, [this, filter = filter](PKResultsStream *stream) {
-            const auto extendingComponents = m_appdata->componentsByExtends(filter.extends);
-            auto resources = resultsByComponents(extendingComponents);
-            stream->sendResources(resources, filter.state != AbstractResource::Broken);
+            m_appdata->componentsByExtends(filter.extends).then(this, [this, stream, filter](const QFuture<AppStream::ComponentBox> &extendingComponents) {
+                auto resources = resultsByComponents(extendingComponents.result());
+                stream->sendResources(resources, filter.state != AbstractResource::Broken);
+            });
         });
     } else if (filter.state == AbstractResource::Upgradeable) {
         return new ResultsStream(QStringLiteral("PackageKitStream-upgradeable"),
@@ -773,19 +772,25 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         });
     } else {
         return deferredResultStream(u"PackageKitStream-search"_s, [this, filter = filter](PKResultsStream *stream) {
-            auto loadComponents = [](const auto &filter, const auto &appdata) -> QCoro::Task<AppStream::ComponentBox> {
-                AppStream::ComponentBox components(AppStream::ComponentBox::FlagNone);
+            auto loadComponents = [](const auto &filter, const auto &appdata) -> QFuture<AppStream::ComponentBox> {
+                QFuture<AppStream::ComponentBox> components;
                 if (!filter.search.isEmpty()) {
                     components = appdata->search(filter.search);
                 } else if (filter.category) {
-                    components = co_await AppStreamUtils::componentsByCategoriesTask(appdata.get(), filter.category, AppStream::Bundle::KindUnknown);
+                    components = AppStreamUtils::componentsByCategoriesTask(appdata.get(), filter.category, AppStream::Bundle::KindUnknown);
                 } else {
                     components = appdata->components();
                 }
-                co_return components;
+                return components;
             };
-            QCoro::connect(loadComponents(filter, m_appdata), this, [this, stream, filter](auto &&components) {
+
+            auto watcher = new QFutureWatcher<AppStream::ComponentBox>();
+            auto futureComponents = loadComponents(filter, m_appdata);
+            watcher->setFuture(futureComponents);
+            connect(watcher, &QFutureWatcher<AppStream::ComponentBox>::finished, watcher, &QObject::deleteLater);
+            connect(watcher, &QFutureWatcher<AppStream::ComponentBox>::finished, stream, [this, stream, filter, futureComponents]() {
                 QSet<QString> ids;
+                AppStream::ComponentBox components = futureComponents.result();
                 kFilterInPlace<AppStream::ComponentBox>(components, [&ids](const AppStream::Component &component) {
                     if (ids.contains(component.id())) {
                         return false;
@@ -926,7 +931,6 @@ Transaction *PackageKitBackend::installApplication(AbstractResource *app)
 
 Transaction *PackageKitBackend::removeApplication(AbstractResource *app)
 {
-    Q_ASSERT(!isFetching());
     if (!qobject_cast<PackageKitResource *>(app)) {
         Q_EMIT passiveMessage(i18n("Cannot remove '%1'", app->name()));
         return nullptr;
@@ -936,7 +940,7 @@ Transaction *PackageKitBackend::removeApplication(AbstractResource *app)
 
 QSet<AbstractResource *> PackageKitBackend::upgradeablePackages() const
 {
-    if (isFetching() || !m_packagesToAdd.isEmpty()) {
+    if (!m_packagesToAdd.isEmpty()) {
         return {};
     }
 
@@ -985,20 +989,13 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
     m_updater->setProgressing(false);
 
     includePackagesToAdd();
-    if (isFetching()) {
-        auto a = new OneTimeAction(
-            [this] {
-                Q_EMIT updatesCountChanged();
-            },
-            this);
-        connect(this, &PackageKitBackend::available, a, &OneTimeAction::trigger);
-    } else {
+    if (!m_isFetching) {
         Q_EMIT updatesCountChanged();
     }
 
     if (!m_updater->isDistroUpgrade() && !PackageKit::Daemon::global()->offline()->upgradeTriggered()) {
         const auto loadDone = [this] {
-            auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata.get());
+            auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata->get());
             if (nextRelease) {
                 foundNewMajorVersion(*nextRelease);
             }
@@ -1006,7 +1003,7 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
         if (m_appdataLoaded) {
             loadDone();
         } else {
-            connect(m_appdata.get(), &AppStream::Pool::loadFinished, this, loadDone);
+            connect(m_appdata.get(), &AppStream::ConcurrentPool::loadFinished, this, loadDone);
         }
     }
 }
@@ -1137,7 +1134,7 @@ AbstractBackendUpdater *PackageKitBackend::backendUpdater() const
 QVector<AbstractResource *> PackageKitBackend::extendedBy(const QString &id) const
 {
     const auto components = m_appdata->componentsByExtends(id);
-    return resourcesByComponents<QVector<AbstractResource *>>(components);
+    return resourcesByComponents<QVector<AbstractResource *>>(components.result());
 }
 
 AbstractReviewsBackend *PackageKitBackend::reviewsBackend() const
@@ -1191,7 +1188,7 @@ void PackageKitBackend::loadAllPackages()
     if (m_allPackagesLoaded) {
         return;
     }
-    const auto components = m_appdata->components();
+    const auto components = m_appdata->components().result();
     for (const auto &component : components) {
         if (!component.packageNames().isEmpty()) {
             addComponent(component);
