@@ -7,10 +7,10 @@
 #include "SnapBackend.h"
 #include "SnapResource.h"
 #include "SnapTransaction.h"
-#include "appstream/AppStreamIntegration.h"
 #include <Category/Category.h>
 #include <Transaction/Transaction.h>
 #include <appstream/AppStreamUtils.h>
+#include <appstream/OdrsReviewsBackend.h>
 #include <resources/SourcesModel.h>
 #include <resources/StandardBackendUpdater.h>
 #include <resources/StoredResultsStream.h>
@@ -82,8 +82,10 @@ private:
 SnapBackend::SnapBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_updater(new StandardBackendUpdater(this))
-    , m_reviews(AppStreamIntegration::global()->reviews())
+    , m_reviews(OdrsReviewsBackend::global())
 {
+    connect(m_updater, &StandardBackendUpdater::updatesCountChanged, this, &SnapBackend::updatesCountChanged);
+
     connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, [this] {
         m_reviews->emitRatingFetched(this, kTransform<QList<AbstractResource *>>(m_resources.values(), [](AbstractResource *r) {
                                          return r;
@@ -138,16 +140,13 @@ ResultsStream *SnapBackend::search(const AbstractResourcesBackend::Filters &filt
 ResultsStream *SnapBackend::findResourceByPackageName(const QUrl &search)
 {
     Q_ASSERT(!search.host().isEmpty() || !AppStreamUtils::appstreamIds(search).isEmpty());
-    return search.scheme() == QLatin1String("snap") ? populate(m_client.find(QSnapdClient::MatchName, search.host())) :
-#ifdef SNAP_FIND_COMMON_ID
-        search.scheme() == QLatin1String("appstream")
+    return search.scheme() == QLatin1String("snap") ? populate(m_client.find(QSnapdClient::MatchName, search.host()))
+        : search.scheme() == QLatin1String("appstream")
         ? populate(kTransform<QVector<QSnapdFindRequest *>>(AppStreamUtils::appstreamIds(search),
                                                             [this](const QString &id) {
                                                                 return m_client.find(QSnapdClient::MatchCommonId, id);
                                                             }))
-        :
-#endif
-        voidStream();
+        : voidStream();
 }
 
 template<class T>
@@ -186,7 +185,7 @@ ResultsStream *SnapBackend::populateJobsWithFilter(const QVector<T *> &jobs, std
     watcher->setFuture(future);
     connect(watcher, &QFutureWatcher<void>::finished, watcher, &QObject::deleteLater);
     connect(watcher, &QFutureWatcher<void>::finished, stream, [this, jobs, filter, stream] {
-        QVector<AbstractResource *> ret;
+        QVector<StreamResult> ret;
         for (auto job : jobs) {
             job->deleteLater();
             if (job->error()) {
@@ -219,16 +218,6 @@ ResultsStream *SnapBackend::populateJobsWithFilter(const QVector<T *> &jobs, std
     return stream;
 }
 
-void SnapBackend::setFetching(bool fetching)
-{
-    if (m_fetching != fetching) {
-        m_fetching = fetching;
-        Q_EMIT fetchingChanged();
-    } else {
-        qWarning() << "fetching already on state" << fetching;
-    }
-}
-
 AbstractBackendUpdater *SnapBackend::backendUpdater() const
 {
     return m_updater;
@@ -257,6 +246,27 @@ Transaction *SnapBackend::removeApplication(AbstractResource *_app)
     return new SnapTransaction(&m_client, app, Transaction::RemoveRole, AbstractResource::None);
 }
 
+void SnapBackend::checkForUpdates()
+{
+    if (m_updatesFetcher) {
+        qWarning() << "Already fetching updates";
+        return;
+    }
+
+    m_updatesFetcher = new StoredResultsStream({populate(m_client.findRefreshable())});
+    connect(m_updatesFetcher, &StoredResultsStream::finishedResources, this, [this](const QVector<StreamResult> &resources) {
+        for (SnapResource *res : std::as_const(m_resources)) {
+            bool contained = kContains(resources, [res](const StreamResult &in) {
+                return in.resource == res;
+            });
+            if (contained) {
+                res->setState(AbstractResource::Upgradeable);
+                res->updateSizes();
+            }
+        }
+    });
+}
+
 QString SnapBackend::displayName() const
 {
     return QStringLiteral("Snap");
@@ -265,14 +275,20 @@ QString SnapBackend::displayName() const
 void SnapBackend::refreshStates()
 {
     auto ret = new StoredResultsStream({populate(m_client.getSnaps())});
-    connect(ret, &StoredResultsStream::finishedResources, this, [this](const QVector<AbstractResource *> &resources) {
-        for (auto res : qAsConst(m_resources)) {
-            if (resources.contains(res))
+    connect(ret, &StoredResultsStream::finishedResources, this, [this](const QVector<StreamResult> &resources) {
+        for (auto res : std::as_const(m_resources)) {
+            bool contained = kContains(resources, [res](const StreamResult &in) {
+                return in.resource == res;
+            });
+            res->updateSizes();
+            if (contained)
                 res->setState(AbstractResource::Installed);
             else
                 res->setState(AbstractResource::None);
         }
+        checkForUpdates();
     });
 }
 
 #include "SnapBackend.moc"
+#include "moc_SnapBackend.cpp"
