@@ -28,6 +28,8 @@
 #include <functional>
 #include <resources/DiscoverAction.h>
 
+using namespace Qt::StringLiterals;
+
 ResourcesModel *ResourcesModel::s_self = nullptr;
 
 ResourcesModel *ResourcesModel::global()
@@ -41,15 +43,12 @@ ResourcesModel *ResourcesModel::global()
 
 ResourcesModel::ResourcesModel(QObject *parent)
     : QObject(parent)
-    , m_isFetching(false)
-    , m_initializingBackendsCount(0)
     , m_currentApplicationBackend(nullptr)
-    , m_allInitializedEmitter(new QTimer(this))
     , m_updatesCount(
           0,
           [this] {
               int ret = 0;
-              for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
+              for (AbstractResourcesBackend *backend : std::as_const(m_backends)) {
                   ret += backend->updatesCount();
               }
               return ret;
@@ -60,20 +59,22 @@ ResourcesModel::ResourcesModel(QObject *parent)
     , m_fetchingUpdatesProgress(
           0,
           [this] {
-              if (m_backends.isEmpty())
+              if (m_backends.isEmpty()) {
                   return 0;
+              }
 
               int sum = 0;
-              for (auto backend : qAsConst(m_backends)) {
-                  sum += backend->fetchingUpdatesProgress();
+              int weights = 0;
+              for (auto backend : std::as_const(m_backends)) {
+                  sum += backend->fetchingUpdatesProgress() * backend->fetchingUpdatesProgressWeight();
+                  weights += backend->fetchingUpdatesProgressWeight();
               }
-              return sum / (int)m_backends.count();
+              return sum / weights;
           },
           [this](int progress) {
               Q_EMIT fetchingUpdatesProgressChanged(progress);
           })
 {
-    connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
     connect(this, &ResourcesModel::backendsChanged, this, &ResourcesModel::initApplicationsBackend);
 }
 
@@ -81,15 +82,6 @@ void ResourcesModel::init(bool load)
 {
     Q_ASSERT(QCoreApplication::instance()->thread() == QThread::currentThread());
 
-    m_allInitializedEmitter->setSingleShot(true);
-    m_allInitializedEmitter->setInterval(0);
-    connect(m_allInitializedEmitter, &QTimer::timeout, this, [this]() {
-        if (m_initializingBackendsCount == 0) {
-            m_isInitializing = false;
-            Q_EMIT allInitialized();
-        }
-    });
-
     if (load) {
         registerAllBackends();
     }
@@ -120,24 +112,33 @@ ResourcesModel::~ResourcesModel()
     qDeleteAll(m_backends);
 }
 
-void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
+void ResourcesModel::addResourcesBackends(const QList<AbstractResourcesBackend *> &backends)
+{
+    auto added = false;
+
+    for (const auto backend : backends) {
+        added |= addResourcesBackend(backend);
+    }
+
+    if (added) {
+        Q_EMIT backendsChanged();
+    }
+}
+
+bool ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
 {
     Q_ASSERT(!m_backends.contains(backend));
     if (!backend->isValid()) {
         qCWarning(LIBDISCOVER_LOG) << "Discarding invalid backend" << backend->name();
         CategoryModel::global()->blacklistPlugin(backend->name());
         backend->deleteLater();
-        return;
+        return false;
     }
 
     m_backends += backend;
-    if (!backend->isFetching()) {
-        m_updatesCount.reevaluate();
-    } else {
-        m_initializingBackendsCount++;
-    }
+    m_updatesCount.reevaluate();
 
-    connect(backend, &AbstractResourcesBackend::fetchingChanged, this, &ResourcesModel::callerFetchingChanged);
+    connect(backend, &AbstractResourcesBackend::contentsChanged, this, &ResourcesModel::callerContentsChanged);
     connect(backend, &AbstractResourcesBackend::allDataChanged, this, &ResourcesModel::updateCaller);
     connect(backend, &AbstractResourcesBackend::resourcesChanged, this, &ResourcesModel::resourceDataChanged);
     connect(backend, &AbstractResourcesBackend::updatesCountChanged, this, [this] {
@@ -149,23 +150,13 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
     connect(backend, &AbstractResourcesBackend::inlineMessageChanged, this, &ResourcesModel::setInlineMessage);
-    connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
-    if (backend->reviewsBackend()) {
-        connect(backend->reviewsBackend(), &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
-    }
-
-    // In case this is in fact the first backend to be added, and also happens to be
-    // pre-filled, we still need for the rest of the backends to be added before trying
-    // to send out the initialized signal. To ensure this happens, schedule it for the
-    // start of the next run of the event loop.
-    if (m_initializingBackendsCount == 0) {
-        m_allInitializedEmitter->start();
-    } else {
-        slotFetching();
+    if (auto reviewsBackend = backend->reviewsBackend()) {
+        connect(reviewsBackend, &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
     }
+    return true;
 }
 
-void ResourcesModel::callerFetchingChanged()
+void ResourcesModel::callerContentsChanged()
 {
     AbstractResourcesBackend *backend = qobject_cast<AbstractResourcesBackend *>(sender());
 
@@ -177,20 +168,8 @@ void ResourcesModel::callerFetchingChanged()
         Q_EMIT backendsChanged();
         CategoryModel::global()->blacklistPlugin(backend->name());
         backend->deleteLater();
-        slotFetching();
         return;
     }
-
-    if (backend->isFetching()) {
-        m_initializingBackendsCount++;
-        slotFetching();
-    } else {
-        m_initializingBackendsCount--;
-        if (m_initializingBackendsCount == 0)
-            m_allInitializedEmitter->start();
-        else
-            slotFetching();
-    }
 }
 
 void ResourcesModel::updateCaller(const QVector<QByteArray> &properties)
@@ -207,13 +186,9 @@ QVector<AbstractResourcesBackend *> ResourcesModel::backends() const
 
 bool ResourcesModel::hasSecurityUpdates() const
 {
-    bool ret = false;
-
-    for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
-        ret |= backend->hasSecurityUpdates();
-    }
-
-    return ret;
+    return std::any_of(m_backends.constBegin(), m_backends.constEnd(), [](const AbstractResourcesBackend *backend) {
+        return backend->hasSecurityUpdates();
+    });
 }
 
 void ResourcesModel::installApplication(AbstractResource *app)
@@ -234,31 +209,15 @@ void ResourcesModel::removeApplication(AbstractResource *app)
 void ResourcesModel::registerAllBackends()
 {
     DiscoverBackendsFactory f;
-    const auto backends = f.allBackends();
-    if (m_initializingBackendsCount == 0 && backends.isEmpty()) {
-        qCWarning(LIBDISCOVER_LOG) << "Couldn't find any backends";
-        m_allInitializedEmitter->start();
-    } else {
-        for (AbstractResourcesBackend *b : backends) {
-            addResourcesBackend(b);
-        }
-        Q_EMIT backendsChanged();
-    }
+    addResourcesBackends(f.allBackends());
+    m_isInitializing = false;
 }
 
 void ResourcesModel::registerBackendByName(const QString &name)
 {
     DiscoverBackendsFactory f;
     const auto backends = f.backend(name);
-    for (auto b : backends)
-        addResourcesBackend(b);
-
-    Q_EMIT backendsChanged();
-}
-
-bool ResourcesModel::isFetching() const
-{
-    return m_isFetching;
+    addResourcesBackends(backends);
 }
 
 bool ResourcesModel::isInitializing() const
@@ -266,25 +225,6 @@ bool ResourcesModel::isInitializing() const
     return m_isInitializing;
 }
 
-void ResourcesModel::slotFetching()
-{
-    bool newFetching = false;
-    for (AbstractResourcesBackend *b : qAsConst(m_backends)) {
-        // isFetching should sort of be enough. However, sometimes the backend itself
-        // will still be operating on things, which from a model point of view would
-        // still mean something going on. So, interpret that as fetching as well, for
-        // the purposes of this property.
-        if (b->isFetching() || (b->backendUpdater() && b->backendUpdater()->isProgressing())) {
-            newFetching = true;
-            break;
-        }
-    }
-    if (newFetching != m_isFetching) {
-        m_isFetching = newFetching;
-        Q_EMIT fetchingChanged(m_isFetching);
-    }
-}
-
 bool ResourcesModel::isBusy() const
 {
     return TransactionModel::global()->rowCount() > 0;
@@ -292,13 +232,9 @@ bool ResourcesModel::isBusy() const
 
 bool ResourcesModel::isExtended(const QString &id)
 {
-    bool ret = true;
-    for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
-        ret = backend->extends().contains(id);
-        if (ret)
-            break;
-    }
-    return ret;
+    return std::any_of(m_backends.constBegin(), m_backends.constEnd(), [&](AbstractResourcesBackend *backend) {
+        return backend->extends(id);
+    });
 }
 
 AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream *> &streams)
@@ -306,11 +242,11 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream *> &st
 {
     Q_ASSERT(!streams.contains(nullptr));
     if (streams.isEmpty()) {
-        qCWarning(LIBDISCOVER_LOG) << "no streams to aggregate!!";
+        qCWarning(LIBDISCOVER_LOG) << "AggregatedResultsStream: No streams to aggregate!";
         QTimer::singleShot(0, this, &AggregatedResultsStream::clear);
     }
 
-    for (auto stream : streams) {
+    for (const auto &stream : streams) {
         connect(stream, &ResultsStream::resourcesFound, this, &AggregatedResultsStream::addResults);
         connect(stream, &QObject::destroyed, this, &AggregatedResultsStream::streamDestruction);
         connect(this, &ResultsStream::fetchMore, stream, &ResultsStream::fetchMore);
@@ -323,12 +259,13 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream *> &st
 
 AggregatedResultsStream::~AggregatedResultsStream() = default;
 
-void AggregatedResultsStream::addResults(const QVector<AbstractResource *> &res)
+void AggregatedResultsStream::addResults(const QVector<StreamResult> &streams)
 {
-    for (auto r : res)
-        connect(r, &QObject::destroyed, this, &AggregatedResultsStream::resourceDestruction);
+    for (const auto &stream : streams) {
+        connect(stream.resource, &QObject::destroyed, this, &AggregatedResultsStream::resourceDestruction);
+    }
 
-    m_results += res;
+    m_results += streams;
 
     m_delayedEmission.start();
 }
@@ -345,7 +282,13 @@ void AggregatedResultsStream::emitResults()
 
 void AggregatedResultsStream::resourceDestruction(QObject *obj)
 {
-    m_results.removeAll(qobject_cast<AbstractResource *>(obj));
+    for (auto it = m_results.begin(); it != m_results.end(); ++it) {
+        if (obj == it->resource) {
+            it = m_results.erase(it);
+        } else {
+            ++it;
+        }
+    }
 }
 
 void AggregatedResultsStream::streamDestruction(QObject *obj)
@@ -377,8 +320,9 @@ AggregatedResultsStream *ResourcesModel::search(const AbstractResourcesBackend::
 
 void ResourcesModel::checkForUpdates()
 {
-    for (auto backend : qAsConst(m_backends))
+    for (const auto backend : std::as_const(m_backends)) {
         backend->checkForUpdates();
+    }
 }
 
 AbstractResourcesBackend *ResourcesModel::currentApplicationBackend() const
@@ -390,14 +334,15 @@ void ResourcesModel::setCurrentApplicationBackend(AbstractResourcesBackend *back
 {
     if (backend != m_currentApplicationBackend) {
         if (write) {
-            KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
-            if (backend)
+            KConfigGroup settings(KSharedConfig::openConfig(), u"ResourcesModel"_s);
+            if (backend) {
                 settings.writeEntry("currentApplicationBackend", backend->name());
-            else
+            } else {
                 settings.deleteEntry("currentApplicationBackend");
+            }
         }
 
-        qCDebug(LIBDISCOVER_LOG) << "setting currentApplicationBackend" << backend;
+        qCDebug(LIBDISCOVER_LOG) << "setting currentApplicationBackend to" << backend;
         m_currentApplicationBackend = backend;
         Q_EMIT currentApplicationBackendChanged(backend);
     }
@@ -407,12 +352,12 @@ void ResourcesModel::initApplicationsBackend()
 {
     const auto name = applicationSourceName();
 
-    auto idx = kIndexOf(m_backends, [name](AbstractResourcesBackend *b) {
-        return b->hasApplications() && b->name() == name;
+    auto idx = kIndexOf(m_backends, [name](AbstractResourcesBackend *backend) {
+        return backend->hasApplications() && backend->name() == name;
     });
     if (idx < 0) {
-        idx = kIndexOf(m_backends, [](AbstractResourcesBackend *b) {
-            return b->hasApplications();
+        idx = kIndexOf(m_backends, [](AbstractResourcesBackend *backend) {
+            return backend->hasApplications();
         });
         qCDebug(LIBDISCOVER_LOG) << "falling back applications backend to" << idx;
     }
@@ -421,10 +366,15 @@ void ResourcesModel::initApplicationsBackend()
 
 QString ResourcesModel::applicationSourceName() const
 {
-    KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+    KConfigGroup settings(KSharedConfig::openConfig(), u"ResourcesModel"_s);
     return settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
 }
 
+QString ResourcesModel::distroName() const
+{
+    return KOSRelease().name();
+}
+
 QUrl ResourcesModel::distroBugReportUrl()
 {
     return QUrl(KOSRelease().bugReportUrl());
@@ -439,3 +389,5 @@ void ResourcesModel::setInlineMessage(const QSharedPointer<InlineMessage> &inlin
     m_inlineMessage = inlineMessage;
     Q_EMIT inlineMessageChanged(inlineMessage);
 }
+
+#include "moc_ResourcesModel.cpp"
