@@ -42,7 +42,7 @@ bool isOnline()
 DiscoverNotifier::DiscoverNotifier(const std::chrono::seconds &checkDelay, QObject *parent)
     : QObject(parent)
 {
-    m_settings = new UpdatesSettings(this);
+    m_settings = std::make_unique<UpdatesSettings>();
     m_settingsWatcher = KConfigWatcher::create(m_settings->sharedConfig());
     QNetworkInformation::loadBackendByFeatures(QNetworkInformation::Feature::Reachability | QNetworkInformation::Feature::TransportMedium);
     if (auto info = QNetworkInformation::instance()) {
@@ -135,24 +135,33 @@ void DiscoverNotifier::showDiscoverUpdates(const QString &xdgActivationToken)
     }
 }
 
-bool DiscoverNotifier::notifyAboutUpdates() const
+bool DiscoverNotifier::checkTriggerTimes(const QDateTime &lastTriggerTime) const
 {
     if (state() != NormalUpdates && state() != SecurityUpdates) {
         // it's not very helpful to notify that everything is in order
-        qCDebug(NOTIFIER) << "Not notifying about updates, state is" << state();
+        qCDebug(NOTIFIER) << "Not triggering, state is" << state();
         return false;
     }
 
     if (m_settings->requiredNotificationInterval() < 0) {
-        qCDebug(NOTIFIER) << "Not notifying about updates, requiredNotificationInterval is" << m_settings->requiredNotificationInterval();
+        qCDebug(NOTIFIER) << "Not triggering, requiredNotificationInterval is" << m_settings->requiredNotificationInterval();
         return false;
     }
 
     // To configure to a random value, execute:
     // kwriteconfig5 --file PlasmaDiscoverUpdates --group Global --key RequiredNotificationInterval 3600
-    const QDateTime earliestNextNotificationTime = m_settings->lastNotificationTime().addSecs(m_settings->requiredNotificationInterval());
-    if (earliestNextNotificationTime.isValid() && earliestNextNotificationTime > QDateTime::currentDateTimeUtc()) {
-        qCDebug(NOTIFIER) << "Not notifying about updates, earliestNextNotificationTime is" << earliestNextNotificationTime;
+    const QDateTime earliestNextTriggerTime = lastTriggerTime.addSecs(m_settings->requiredNotificationInterval());
+    if (earliestNextTriggerTime.isValid() && earliestNextTriggerTime > QDateTime::currentDateTimeUtc()) {
+        qCDebug(NOTIFIER) << "Not triggering, earliestNextTriggerTime is" << earliestNextTriggerTime;
+        return false;
+    }
+
+    return true;
+}
+
+bool DiscoverNotifier::notifyAboutUpdates() const
+{
+    if (!checkTriggerTimes(m_settings->lastNotificationTime())) {
         return false;
     }
 
@@ -166,6 +175,19 @@ bool DiscoverNotifier::notifyAboutUpdates() const
     return true;
 }
 
+bool DiscoverNotifier::proceedUnattended() const
+{
+    if (!checkTriggerTimes(m_settings->lastUnattendedTrigger())) {
+        return false;
+    }
+
+    if (QDBusConnection::sessionBus().interface()->isServiceRegistered(QStringLiteral("org.kde.discover"))) {
+        qCDebug(NOTIFIER) << "Not proceeding with unattended update, discover is running";
+        return false;
+    }
+    return true;
+}
+
 void DiscoverNotifier::showUpdatesNotification()
 {
     if (m_updatesAvailableNotification) {
@@ -236,11 +258,40 @@ static bool isConnectionAdequate()
     }
 }
 
+bool DiscoverNotifier::isSystemUpdateable() const
+{
+    const bool updateable = !m_isBusy && isOnline() && (m_hasSecurityUpdates || m_hasUpdates);
+    qCDebug(NOTIFIER) << "isSystemUpdateable:" << updateable << "isBusy:" << m_isBusy << "isOnline:" << isOnline() << "securityUpdates:" << m_hasSecurityUpdates
+                      << "otherUpdates:" << m_hasUpdates;
+    return updateable;
+}
+
+void DiscoverNotifier::startUnattendedUpdates()
+{
+    auto process = new QProcess(this);
+    connect(process, &QProcess::errorOccurred, this, [](QProcess::ProcessError error) {
+        qWarning() << "Error running plasma-discover" << error;
+    });
+    connect(process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [this, process](int exitCode, QProcess::ExitStatus exitStatus) {
+        qDebug() << "Finished running plasma-discover" << exitCode << exitStatus;
+        process->deleteLater();
+        if (exitCode == 0) {
+            settings()->setLastUnattendedTrigger(QDateTime::currentDateTimeUtc());
+        }
+        settings()->save();
+        setBusy(false);
+    });
+
+    setBusy(true);
+    process->start(QStringLiteral("plasma-discover"), {QStringLiteral("--headless-update")});
+    qInfo() << "started unattended update" << QDateTime::currentDateTimeUtc();
+}
+
 void DiscoverNotifier::refreshUnattended()
 {
     m_settings->read();
 
-    if (!notifyAboutUpdates()) {
+    if (!proceedUnattended()) {
         qCDebug(NOTIFIER) << "refreshUnattended: not notifying about updates";
         return;
     }
@@ -253,11 +304,10 @@ void DiscoverNotifier::refreshUnattended()
 
     if (enabled) {
         qCDebug(NOTIFIER) << "Enabling unattended updates";
-        m_unattended = new UnattendedUpdates(this);
+        m_unattended = std::make_unique<UnattendedUpdates>(this);
     } else {
         qCDebug(NOTIFIER) << "Disabling unattended updates";
-        delete m_unattended;
-        m_unattended = nullptr;
+        m_unattended.reset();
     }
 }
 
@@ -321,7 +371,7 @@ void DiscoverNotifier::recheckSystemUpdateNeeded()
     for (BackendNotifierModule *module : std::as_const(m_backends))
         module->recheckSystemUpdateNeeded();
 
-    refreshUnattended();
+    QTimer::singleShot(20000, this, &DiscoverNotifier::refreshUnattended);
 }
 
 QStringList DiscoverNotifier::loadedModules() const
